package lbms.plugins.mldht.kad;

import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.UnknownHostException;
import java.nio.ByteBuffer;
import java.util.List;

/**
 * @author Damokles
 *
 */
public class PackUtil {

	/**
	 * Packs a BucketEntry into the provided buffer.
	 *
	 * @param e Entry to insert
	 * @param buffer must be at least 26bytes per Entry
	 * @param off Offset to use
	 * @throws IllegalArgumentException if buffer is too small
	 */
	public static void PackBucketEntry (KBucketEntry e, byte[] buffer, int off) {
		// first check size
		if (off + 26 > buffer.length) {
			throw new IllegalArgumentException("Not enough room in buffer");
		}
		ByteBuffer bb = ByteBuffer.wrap(buffer, off, 26);

		//Uint8* data = (Uint8*)ba.data();
		//Uint8* ptr = data + off;

		InetSocketAddress addr = e.getAddress();
		// copy ID, IP address and port into the buffer
		bb.put(e.getID().getHash());
		bb.put(addr.getAddress().getAddress());
		//bt::WriteUint32(ptr,20,addr.ipAddress().IPv4Addr());
		bb.putShort((short) addr.getPort());
	}

	/**
	 * Unpacks a Entry from a byte array
	 *
	 * @param buffer byte array with serialized entry
	 * @param off the Offset to use
	 * @return deserialized Entry
	 * @throws IllegalArgumentException if buffer is to small
	 */
	public static KBucketEntry UnpackBucketEntry (byte[] buffer, int off) {
		if (off + 26 > buffer.length) {
			throw new IllegalArgumentException("Not enough room in buffer");
		}
		ByteBuffer bb = ByteBuffer.wrap(buffer, off, 26);

		byte[] key = new byte[20];
		bb.get(key);

		byte[] inetaddr = new byte[4];
		bb.get(inetaddr);

		InetSocketAddress addr = null;
		//UnknownHostException shouldn't occur since IP is provided
		try {

			addr = new InetSocketAddress(InetAddress.getByAddress(inetaddr),
					bb.getShort() & 0xFFFF); // & 0xFFFF is used to get an unsigned short
		} catch (UnknownHostException e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}

		return new KBucketEntry(addr, new Key(key), 0);
	}

	public static DBItem[] UnpackDBItems (byte[] buffer) {
		int count = buffer.length / 6;
		DBItem[] result = new DBItem[count];
		for (int i = 0; i< count;i++) {
			result[i] = new DBItem(buffer, 6*i);
		}
		return result;
	}

	public static byte[] PackDBItems (List<DBItem> items) {
		if(items == null)
			return new byte[0];
		byte[] result = new byte[items.size()*6];
		for (int i = 0;i<items.size();i++) {
			System.arraycopy(items.get(i).getData(), 0, result, i*6, 6);
		}
		return result;
	}
}
